package com.mortals.xhx.module.skin.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.PageInfo;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.key.Constant;
import com.mortals.xhx.common.utils.FreeMarkerUtils;
import com.mortals.xhx.module.skin.model.*;
import com.mortals.xhx.module.skin.service.SkinFieldService;
import freemarker.template.Template;
import org.apache.http.entity.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.skin.dao.SkinBaseDao;
import com.mortals.xhx.module.skin.service.SkinBaseService;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * SkinBaseService
 * 系统基础皮肤 service实现
 *
 * @author zxfei
 * @date 2022-05-24
 */
@Service("skinBaseService")
public class SkinBaseServiceImpl extends AbstractCRUDServiceImpl<SkinBaseDao, SkinBaseEntity, Long> implements SkinBaseService {

    @Autowired
    private SkinFieldService skinFieldService;
    @Autowired
    private UploadService uploadService;

    @Override
    protected void validData(SkinBaseEntity entity, Context context) throws AppException {
        SkinBaseQuery skinBaseQuery = new SkinBaseQuery();
        skinBaseQuery.setName(entity.getName());
        SkinBaseEntity skinBaseEntity = this.selectOne(skinBaseQuery);
        if (!ObjectUtils.isEmpty(skinBaseEntity) && entity.newEntity()) {
            throw new AppException("皮肤名称已存在");
        }
        super.validData(entity, context);
    }

    @Override
    protected void findAfter(SkinBaseEntity params, Context context, List<SkinBaseEntity> list) throws AppException {
        list.stream().forEach(item -> {
            List<SkinFieldEntity> skinFieldEntities = skinFieldService.find(new SkinFieldQuery().skinId(item.getId()));
            item.setSkinFieldList(skinFieldEntities);
        });
        super.findAfter(params, context, list);
    }


    @Override
    protected void saveAfter(SkinBaseEntity entity, Context context) throws AppException {
        //删除关联后新增
        saveSkinFields(entity, context);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(SkinBaseEntity entity, Context context) throws AppException {
        saveSkinFields(entity, context);
        super.updateBefore(entity, context);
    }

    private void saveSkinFields(SkinBaseEntity entity, Context context) {
        Long[] ids = skinFieldService.find(new SkinFieldQuery().skinId(entity.getId())).stream().map(item -> item.getId()).toArray(Long[]::new);
        if (!ObjectUtils.isEmpty(ids)) {
            skinFieldService.remove(ids, context);
        }
        if (!ObjectUtils.isEmpty(entity.getSkinFieldList())) {
            entity.getSkinFieldList().stream().forEach(item -> {
                item.setSkinId(entity.getId());
            });
            skinFieldService.save(entity.getSkinFieldList(), context);
        }
    }

    @Override
    protected void updateBefore(SkinBaseEntity entity, Context context) throws AppException {
        genTemplateCss(entity, context);
        super.updateBefore(entity, context);
    }

    @Override
    protected void saveBefore(SkinBaseEntity entity, Context context) throws AppException {
        //todo 合成实现css模板文件
        genTemplateCss(entity, context);
        super.saveBefore(entity, context);
    }


    private void genTemplateCss(SkinBaseEntity entity, Context context) {
        //加载模板，合成模板数据
        Map data = new HashMap();
        data.put("columns", entity.getSkinFieldList());
        String temp = "skin.css.ftl";
        try {
            FreeMarkerUtils.initFreeMarker(temp);
            StringWriter sw = new StringWriter();
            Template template = FreeMarkerUtils.getTemplate(temp);
            template.process(data, sw);

            InputStream inputStream = new ByteArrayInputStream(sw.toString().getBytes(Constant.UTF8));
            MultipartFile file = new MockMultipartFile("file.css", "file.css", ContentType.APPLICATION_OCTET_STREAM.toString(), inputStream);
            String filePath = uploadService.saveFileUpload(file, "/file/uploadfile/", context.getUser());
            entity.setCssFilePath(filePath);

        } catch (Exception e) {
            log.error("css失败：", e);

        }

    }
}